// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (C) 2023 Deepcomputing Technology Co., Ltd.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "buf.h"

#define PROG_NAME "K1X-EC-COMM"

//#define DEBUG 1

int bufInit(buf_t *pbuf, unsigned int nsize)
{
    pbuf->buf = (char*)malloc(nsize);
    if (pbuf->buf == NULL) {
        return 1;
    }
    pbuf->RIdx = 0;
    pbuf->WIdx = 0;
    pbuf->EndIdx = pbuf->nsize - 1;
    pbuf->nsize = nsize;
    pthread_mutex_init(&pbuf->mtx,NULL);
    return 0;
}

int bufGetWriteBufAddr(buf_t *pbuf, char **addr, unsigned int *pnsize)
{
    unsigned int RIdx = pbuf->RIdx;
    unsigned int nleft;

    if (RIdx > pbuf->WIdx) {
        nleft = RIdx - pbuf->WIdx;
    } else {
        nleft = pbuf->nsize - pbuf->WIdx;
    }
     
    *addr = pbuf->buf + pbuf->WIdx;
    *pnsize = nleft;

#ifdef DEBUG
    printf("write w=%x, r=%x, d=%x\n", pbuf->WIdx, RIdx, nleft);
#endif

    return 0;
}

/*End buf too small, skip it*/
int bufSkipBuf(buf_t *pbuf, unsigned int nsize)
{ 
    if (nsize  + pbuf->WIdx >= pbuf->nsize) {
#ifdef DEBUG
        printf("SkipB w=%x, d=%x\n", pbuf->WIdx, nsize);
#endif
        pbuf->EndIdx = pbuf->WIdx;
        pbuf->WIdx = 0;
    } else {
        /*do nothing*/
    }

    return 0;
}

/*write data*/
int bufPushData(buf_t *pbuf, unsigned int nsize)
{
    pbuf->WIdx += nsize;
    if (pbuf->WIdx >= pbuf->nsize) {
        pbuf->WIdx = 0;
        pbuf->EndIdx = pbuf->nsize - 1;
    }

#ifdef DEBUG
    printf("Push w=%x, d=%x\n", pbuf->WIdx, nsize);
#endif

    return 0;
}

int bufGetReadBufAddr(buf_t *pbuf, char **addr, unsigned int *pnsize)
{
    int WIdx = pbuf->WIdx;
    int nleft;

    if (WIdx >= pbuf->RIdx) {
        nleft = WIdx - pbuf->RIdx;
    } else {
        nleft = pbuf->EndIdx + 1 - pbuf->RIdx;
    }
     
    *addr = pbuf->buf + pbuf->RIdx;
    *pnsize = nleft;

#ifdef DEBUG
    printf("read w=%x, r=%x, d=%x\n", WIdx, pbuf->RIdx, nleft);
#endif

    return 0;
}

/*read data*/
int bufSkipData(buf_t *pbuf, unsigned int nsize)
{
    pbuf->RIdx += nsize;
    if (pbuf->RIdx >= pbuf->EndIdx) {
        pbuf->RIdx = 0;
        pbuf->EndIdx = pbuf->nsize - 1;
    }

#ifdef DEBUG
    printf("skipD r=%x, d=%x\n", pbuf->RIdx, nsize);
#endif

    return 0;
}

int bufClean(buf_t *pbuf)
{
    return 0;
}

int bufDeinit(buf_t *pbuf)
{
    if (pbuf->buf != NULL) {
        free(pbuf->buf);
        pbuf->buf = NULL;
    }

    pthread_mutex_destroy(&pbuf->mtx);
    return 0;
}
