// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (C) 2023 Deepcomputing Technology Co., Ltd.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ec_mgr_uart_msg.h"

#define PROG_NAME "K1X-UART-KEY"

#define START_LINE "EC+"
#define END_LINE "\n"

static char * message_contents[MESSAGE_TYPE_MAX] = {
    "PowerOff",
    "OK",
    "fail",
    "led_power",
    "led_charge"
};

char* led_ctl_contents[] = {
    "off",
    "on",
    "blink",
    "breath",
    "release",
};

int parse_message(char * buf, unsigned int nLen, K1X_meesage_data_t *pdata, unsigned int *pnskip)
{
    char * p;
    char * q;
    char * x;
    K1X_message_type i;

    /*find START_LINE*/
    p = strstr(buf, START_LINE);
    if (p == NULL) {
        //printf("[%s %s] not find START_LINE\n", PROG_NAME, __func__);
        *pnskip = nLen; /*skip all*/
        return 1;
    }

    pdata->type = MESSAGE_TYPE_UNKNOWN;
    p += strlen(START_LINE);
    p += 1;
    for (i = 0; i < (int)MESSAGE_TYPE_MAX; i++) {
        if ((NULL == message_contents[i]) || (strncmp(p, message_contents[i], strlen(message_contents[i])) != 0)) {
            continue;
        }
        q = p + strlen(message_contents[i]);
        x = strstr(q, END_LINE);
        /*check end line*/
        if (x == NULL) {
            printf("[%s %s] not find END_LINE\n", PROG_NAME, __func__);
            *pnskip = q - buf;
            return 1;
        }
        *pnskip = (x - buf) + strlen(END_LINE);
        pdata->type = i;

        switch(pdata->type){
            case MESSAGE_TYPE_OK:
            case MESSAGE_TYPE_FAIL:
            case MESSAGE_TYPE_POWEROFF:
                /*do nothing*/
                break;
            default:
                printf("[%s %s] No support parse type %d\n", PROG_NAME, __func__, pdata->type);
                return 1;
                break;
        }
    }

    if (pdata->type == MESSAGE_TYPE_UNKNOWN) {
        *pnskip = p - buf + strlen(START_LINE);
        return 1;
    }

    return 0;
}

int build_message(char* buf, unsigned int nLen, K1X_meesage_data_t* pdata)
{
    if (buf == NULL || pdata == NULL) {
        printf("[%s]build_message fail!\n", PROG_NAME);
        return 1;
    }

    switch(pdata->type) {
        case MESSAGE_TYPE_OK:
        case MESSAGE_TYPE_FAIL:
        case MESSAGE_TYPE_POWEROFF:
            //sprintf(buf, "%s%s%s", START_LINE, message_contents[pdata->type], END_LINE);
            break;

        case MESSAGE_TYPE_LED_POWER:
        case MESSAGE_TYPE_LED_CHARGE:
            sprintf(buf, "%s %s%s", message_contents[pdata->type], led_ctl_contents[pdata->value], END_LINE);
            break;
        default:
            printf("[%s]No support build type %d\n", PROG_NAME, pdata->type);
            return 1;
            break;
    }

    return 0;
}
