#include <gio/gio.h>  
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>

#define FIFO_PATH "/tmp/upower_fifo"

static void on_signal(GDBusConnection *connection, const gchar *sender_name,  
                      const gchar *object_path, const gchar *interface_name,  
                      const gchar *signal_name, GVariant *parameters,  
                      gpointer user_data) {  
    gchar *s;
    s = g_variant_print (parameters, TRUE);
    g_debug ("%s\n", s);
    g_debug("upower fifo write");
    gint fd_write = open(FIFO_PATH, O_WRONLY);
    if (fd_write == -1) {
	    g_error("can't open upower_fifo");
	    g_free (s);
	    close(fd_write);
        return;
    }

    gint bytes_written = write(fd_write, s, strlen(s) + 1);
    if (bytes_written == -1) {
        g_error("can't write upower_fifo");
	g_free (s);
	close(fd_write);
        return;
    }

    g_debug("upower fifo write end");
    close(fd_write);
    g_free (s);

}  

// 函数用于替换字符串中的字符  
void replaceChar(char *str, char from, char to) {
    // 遍历字符串直到遇到字符串结束符'\0'  
    while (*str) {
        // 如果当前字符是需要被替换的字符  
        if (*str == from) {
            *str = to; // 替换字符  
        }
        str++; // 移动到下一个字符  
    }
}

int main(int argc, char *argv[]) {  
    GDBusConnection *connection;  
    GError *error = NULL;  
    gchar object_path_battery[70] = "/org/freedesktop/UPower/devices/battery_";

    DIR *dir;
    struct dirent *entry;

    dir = opendir("/sys/class/power_supply/");
    if (dir == NULL) {
        perror("opendir() error");
        return 1;
    }

    while ((entry = readdir(dir)) != NULL) {
        if((strncmp(entry->d_name, "sbs", 3) == 0) && (strncmp(entry->d_name, "sbs-charger", 11) != 0)) {
            replaceChar(entry->d_name, '-', '_');
            strcat(object_path_battery, entry->d_name);
            g_debug("%s\n", object_path_battery);
        }
    }

    closedir(dir);


    // 获取 session 总线连接  
    connection = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);  
    if (connection == NULL) {  
        g_printerr("Unable to get session bus: %s\n", error->message);  
        g_error_free(error);  
        return 1;  
    }  
  
    // 假设你想要监控名为 "org.example.MyService" 的服务在 "/org/example/MyService/Object" 路径上的 "MySignal" 信号  
    const gchar *service_name = "org.freedesktop.UPower";  
    const gchar *object_path_lid = "/org/freedesktop/UPower"; 
    const gchar *interface_name = "org.freedesktop.DBus.Properties";  
    const gchar *signal_name = "PropertiesChanged";  
  
    // 添加信号匹配规则并设置回调函数  
    guint rule_id_battery = g_dbus_connection_signal_subscribe(connection,  
                                                      service_name, // 可以为 NULL 以匹配所有服务  
                                                      NULL,  // 可以为 NULL 或 "org.freedesktop.DBus.ObjectManager" 以匹配所有对象  
                                                      NULL, // 可以为 NULL 以匹配所有接口  
                                                      object_path_battery,    // 可以为 NULL 以匹配所有信号  
                                                      NULL,           // 匹配参数，可以为 NULL  
                                                      G_DBUS_SIGNAL_FLAGS_NONE,  
                                                      on_signal,  
                                                      NULL,  
                                                      NULL);  
 
     guint rule_id_lid = g_dbus_connection_signal_subscribe(connection,
                                                      service_name, // 可以为 NULL 以匹配所有服务  
                                                      NULL,  // 可以为 NULL 或 "org.freedesktop.DBus.ObjectManager" 以匹配所有对象  
                                                      NULL, // 可以为 NULL 以匹配所有接口  
                                                      object_path_lid,    // 可以为 NULL 以匹配所有信号  
                                                      NULL,           // 匹配参数，可以为 NULL  
                                                      G_DBUS_SIGNAL_FLAGS_NONE,
                                                      on_signal,
                                                      NULL,
                                                      NULL);


    g_error("start upower monitor");
    // 进入主循环  
    GMainLoop *loop = g_main_loop_new(NULL, FALSE);  
    g_main_loop_run(loop);  
  
    // 当主循环结束时，取消信号订阅  
    g_dbus_connection_signal_unsubscribe(connection, rule_id_battery);  
    g_dbus_connection_signal_unsubscribe(connection, rule_id_lid);

    // 关闭连接  
    g_object_unref(connection);  
  
    return 0;  
}
